module Ach
  class Parser
    def self.parse_file(observer)
      raise Exception.new "No file supplied. Cannot proceed!" if observer.filename.nil?
      File.open(observer.filename).each do |l|
        case l[0].chr
        when "1"
          file_header = FileHeaderRecord.decode(l)
          observer.file_header(file_header)
        when "5"
          batch_header = BatchHeaderRecord.decode(l)
          observer.batch_header(batch_header)
        when "6"
          entry_detail = EntryDetailRecord.decode(l)
          observer.entry_detail(entry_detail)
        when "8"
          batch_control = BatchControlRecord.decode(l)
          observer.batch_control(batch_control)
        when "9"
          unless (l.count "9") == 94
            file_control = FileControlRecord.decode(l)
            observer.file_control(file_control)
          end
        else
        end
      end
    end
  end

  class FileHeaderRecord
    def self.decode(record)
      fh = {}
      fh[:record_type] = record[0].chr
      fh[:priority_code] = record[1,2]
      fh[:wellsfargo_routing_number] = record[4,9]
      fh[:file_id] = record[13,10]
      fh[:file_creation_date] = record[23,6]
      fh[:file_creation_time] = record[29,4]
      fh[:file_id_modifier] = record[33,1]
      fh[:record_size] = record[34,3]
      fh[:blocking_factor] = record[37,2]
      fh[:format_code] = record[39,1]
      fh[:origination_bank] = record[40,23]
      fh[:company_name] = record[63,23]
      return fh
    end
  end

  class BatchHeaderRecord
    def self.decode(record)
      bh = {}
      bh[:record_type] = record[0].chr
      bh[:service_class_code] = record[1,3]
      bh[:company_name] = record[4,16]
      bh[:company_discretionary_data] = record[20,20]
      bh[:company_id] = record[40,10]
      bh[:standard_entry_class_code] = record[50,3]
      bh[:company_entry_description] = record[53,10]
      bh[:company_descriptive_date] = record[63,6]
      bh[:effective_entry_date] = record[69,6]
      bh[:settlement_date] = record[75,3]
      bh[:originator_status_code] = record[78,1]
      bh[:wellsfargo_routing_number] = record[79,8]
      bh[:batch_number] = record[87,7]
      return bh
    end
  end

  class EntryDetailRecord
    def self.decode(record)
      ed = {}
      ed[:record_type] = record[0].chr
      ed[:transaction_code] = record[1,2]
      ed[:receiving_dfi_routing_number] = record[3,8]
      ed[:routing_number_check_digit] = record[11,1]
      ed[:receiving_dfi_account_number] = record[12,17]
      ed[:amount] = record[29,10]
      ed[:individual_id] = record[39,15]
      ed[:individual_name] = record[54,22]
      ed[:discretionary_data] = record[76,2]
      ed[:addenda_record_indicator] = record[78,1]
      ed[:trace_number] = record[79,15]
      return ed
    end
  end

  class BatchControlRecord
    def self.decode(record)
      bcr = {}
      bcr[:record_type] = record[0].chr
      bcr[:service_class_code] = record[1,3]
      bcr[:entry_addenda_count] = record[4,6]
      bcr[:entry_hash] = record[10,10]
      bcr[:total_batch_debit_entry_dollar_amount] = record[20,12]
      bcr[:total_batch_credit_entry_dollar_amount] = record[32,12]
      bcr[:company_id] = record[44,10]
      bcr[:message_authentication_code] = record[54,19]
      bcr[:blank] = record[73,6]
      bcr[:wellsfargo_routing_number] = record[79,8]
      bcr[:batch_number] = record[87,7]
      return bcr
    end
  end

  class FileControlRecord
    def self.decode(record)
      fcr = {}
      msg_codes = []
      msg_codes << record[56,2] unless record[56,2].match(/\s/)
      msg_codes << record [58,2] unless record[58,2].match(/\s/)
      msg_codes << record[60,2] unless record[60,2].match(/\s/)
      fcr[:record_type] = record[0].chr
      fcr[:batch_count] = record[1,6]
      fcr[:block_count] = record[7,6]
      fcr[:entry_addenda_record_count] = record[13,8]
      fcr[:entry_hash_total] = record[21,10]
      fcr[:total_file_debit_entry_amount] = record[31,12]
      fcr[:total_file_credit_entry_amount] = record[43,12]
      fcr[:message_codes] = msg_codes unless msg_codes.empty?
      return fcr
    end
  end

end